'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _actions = require('./actions');

var actions = _interopRequireWildcard(_actions);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var createInitialState = function createInitialState(_ref) {
    var state = _ref.state,
        slice = _ref.slice,
        val = _ref.val,
        immutable = _ref.immutable;

    if (immutable) {
        state = state.set(slice, require('immutable').fromJS(val));
    } else {
        state[slice] = val;
    }
    return state;
};

var scrollToHash = function scrollToHash(hash) {
    if (hash && window && typeof window.requestAnimationFrame === 'function') {
        window.requestAnimationFrame(function () {
            var node = document.getElementById(location.hash.substr(1));
            if (node) {
                node.scrollIntoView();
            }
        });
    }
};

exports.default = function (_ref2) {
    var history = _ref2.history,
        slice = _ref2.slice,
        locationParser = _ref2.locationParser,
        immutable = _ref2.immutable;
    return function (next) {
        return function (reducer, initialState, enhancer) {

            // boilerplate
            if (typeof initialState === 'function' && typeof enhancer === 'undefined') {
                enhancer = initialState;
                initialState = undefined;
            }
            var newInitialState = initialState || enhancer;

            var initialLocation = locationParser(history.location);

            scrollToHash(initialLocation.hash);

            newInitialState = createInitialState({ state: newInitialState, val: initialLocation, slice: slice, immutable: immutable });

            var store = next(reducer, newInitialState, enhancer);

            history.listen(function (location) {
                if (location.silent) return;

                store.dispatch(actions.locationChange(location));

                scrollToHash(location.hash);
            });

            return store;
        };
    };
};